<?php
class ModelExtensionPaymentBifit extends Model
{
	public function getMethod($address, $total)
	{
		return array();
	}

	function __formatprc($price)
	{
		$this->load->model('localisation/currency');
		$shop_currency = $this->config->get('config_currency');
		$decimal_place = $this->currency->getDecimalPlace($shop_currency);

		// форматируем цену согласно настроек
		$price = $decimal_place < 0 ? round($price, $decimal_place) : number_format($price, $decimal_place, '.', '');

		return $price;
	}

	function __checkInn($inn)
	{
		// только цифры в количестве 10 или 12 шт.
		if (!preg_match("/^\d{12}$/", $inn)) {
			$inn = "ИНН задан в неверном формате!";
		}

		return $inn;
	}

	function __checkPhone($tel)
	{
		// удаляем лишние символы
		$phone = str_replace(array('+', '(', ')', '-', ' '), '', trim($tel));

		// получаем первую цифру
		$first = substr($phone, "0", 1);

		// если номер из 10 цифр и первая цифра не 7, то добавляем 7
		$phone = strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;

		// окончательная проверка
		if (!preg_match("/^[0-9]{11}+$/", $phone)) {
			$phone = "Телефон задан в неверном формате!";
		}

		return $phone;
	}

	function __checkEmail($email)
	{
		// проверка модели записи
		if (!preg_match("/^\+\d+$|^\S+@\S+$/", $email)) {
			$email = "E-mail задан в неверном формате!";
		}

		return $email;
	}

	function __checkProductPrice($item_price, $order_subtotal, $order_discount = 0, $order_payment = 0)
	{
		// если скидка больше нуля
		if ($order_discount > 0) {

			// вычисляем процент скидки
			$percent 	= $order_discount * 100 / $order_subtotal;

			// уменьшаем цену товара согласно скидки
			$item_price = $item_price - ($item_price * ($percent / 100));
		}

		// если система оплаты больше нуля
		if ($order_payment > 0) {

			// вычисляем процент наценки
			$percent 	= $order_payment * 100 / ($order_subtotal - $order_discount);

			// увеличиваем цену товара согласно наценки
			$item_price = $item_price + ($item_price * ($percent / 100));
		}

		return $item_price;
	}

	function __loadData($url, $headers = false, $data = false)
	{
		// инициализируем сеанс cURL
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);

		// добавляем данные
		if ($data) {
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		// возврат результата в качестве строки
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		// добавляем headers
		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		// получаем ответ
		$output = curl_exec($ch);

		// завершаем сеанс cURL
		curl_close($ch);

		return $output;
	}

	function __createLog($test, $bifit, $receipt, $receipt_id, $doc, $receipt_link, $loc, $logObj)
	{
		// выводим место и свойство операции
		$loc = $loc == 'admin' ? ' (админ)' : ' (фронт)';
		$log = '####### Боевой запрос' . $loc . "\n";
		$log = $test != '' ? '####### Тестовый запрос' . $loc . "\n" : $log;

		// ОТВЕТ #1

		$log.= '####### Ответ #1: ' . "\n";

		// данные ответа #1
		if (is_object($bifit)) {
			foreach ($bifit as $key=>$val) {
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// ЧЕК

		// сформированный чек для отправки
		$log .= '####### Чек: ' . "\n" . $receipt . "\n";

		// ОТВЕТ #2

		$log .= '####### Ответ #2: ' . "\n";

		// данные ответа #2
		if (is_object($receipt_id)) {
			foreach ($receipt_id as $key=>$val) {
				if (is_array($val)) {
					foreach ($val as $k1=>$v1) {
						if (is_object($val[$k1])) {
							foreach ($val[$k1] as $k2=>$v2) {
								$log .= $v2 != '' ? ' – ' . $k2 . ': ' . $v2 . "\n" : '';
							}
						}
					}
				} else {
					$log .= $val != '' ? $key . ': ' . $val . "\n" : '';
				}
			}
		} else {
			$log .= $receipt_id . "\n";
		}

		// ОТВЕТ #3

		$log .= '####### Ответ #3: ' . "\n";
		$a_3  = '';

		// данные ответа #3
		if (is_object($doc)) {
			foreach ($doc as $key => $val) {
				if (is_object($val)) {
					foreach ($val as $k1 => $v1) {
						if (is_array($v1)) {
							foreach ($v1 as $k2 => $v2) {
								if (is_object($v1[$k2])) {
									$log .= "\n";
									$a_3 .= "\n";
									foreach ($v1[$k2] as $k3 => $v3) {
										$log .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
										$a_3 .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
									}
								}
							}
							$log .= "\n";
							$a_3 .= "\n";
						} elseif (is_object($v1)) {
							foreach ($v1 as $k2 => $v2) {
								if (is_object($v2)) {
									foreach ($v2 as $k3 => $v3) {
										$log .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
										$a_3 .= $v3 != '' ? '   - ' . $k3 . ': ' . $v3 . "\n" : '';
									}
								} else {
									$log .= $v2 != '' ? '  - ' . $k2 . ': ' . $v2 . "\n" : '';
									$a_3 .= $v2 != '' ? '  - ' . $k2 . ': ' . $v2 . "\n" : '';
								}
							}
						} else {
							$log .= $v1 != '' ? ' - ' . $k1 . ': ' . $v1 . "\n" : '';
							$a_3 .= $v1 != '' ? ' - ' . $k1 . ': ' . $v1 . "\n" : '';
						}
					}
				} else {
					$log .= $val != '' ? $key . ': ' . $val . "\n" : '';
					$a_3 .= $val != '' ? $key . ': ' . $val . "\n" : '';
				}
			}
		}

		// ССЫЛКА НА ЧЕК

		// сформированная ссылка на чек
		$log .= '####### Ссылка на чек: ' . "\n" . $receipt_link;

		// параметры операции

		// сохранение
		self::__saveTransaction($logObj, $log, $a_3);
	}

	function __saveTransaction($all_data, $log, $doc)
	{
		$this->db->query("INSERT INTO `" . DB_PREFIX . "bifit_transaction` SET 
		`order_id` = '" . (int)$all_data->order_id . "',
		`cashier` = '" . $all_data->cashier . "',
		`client` = '" . $all_data->client . "',  
		`receipt_link` = '" . $all_data->receipt_link . "', 
		`receipt_id` = '" . $all_data->receipt_id . "', 
		`doc` = '" . $this->db->escape($doc) . "',
		`log` = '" . $this->db->escape($log) . "', 
		`receipt_type` = '" . $all_data->receipt_type . "', 
		`payment_type` = '" . $all_data->payment_type . "',
		`date` = NOW()");
	}

	function __mainProcess($order, $test, $token, $log_ok, $loc)
	{
		// 1. АВТОРИЗАЦИЯ
		//    процесс необходимый для подключения клиента к фискальному процессингу

		// адрес для POST-запроса
		$url = 'https://fp' . $test . '.bifit.com/processing-api/oauth/token';

		// данные ссылки
		$data  = 'token=' . $token;
		$data .= '&client_id=processing-connector-token';
		$data .= '&client_secret=processing-connector-token';
		$data .= '&grant_type=token';

		// headers
		$headers = array('Content-Type: application/x-www-form-urlencoded');

		// декодируем полученный ответ
		$bifit 	 = json_decode(self::__loadData($url, $headers, $data));

		// 2. ФОРМИРОВАНИЕ КОНТЕНТА ЧЕКА
		//    формирование информации для дальнейшей отправки на фискализацию

		// параметры плагина
		$cashier_name 		= $this->config->get('payment_bifit_cashier_name');
		$cashier_inn 		= $this->config->get('payment_bifit_cashier_inn');
		$receipt_type 		= $this->config->get('payment_bifit_receipt_type');
		$tax_system 		= $this->config->get('payment_bifit_tax_system');
		$calc_method 		= $this->config->get('payment_bifit_calculation_method');
		$vat_mode 			= $this->config->get('payment_bifit_vat_mode');
		$vat 				= $this->config->get('payment_bifit_vat');
		$payment_type 		= $this->config->get('payment_bifit_payment_type');
		$payment_address 	= $this->config->get('payment_bifit_payment_address');
		$address_type 		= $this->config->get('payment_bifit_address_type');
		$fio 				= $this->config->get('payment_bifit_fio');

		// адрес клиента в чеке
		if ($address_type && $order["telephone"]) {
			$address = self::__checkPhone($order["telephone"]);
		} else {
			$address = $order["email"];
		}

		// позиции оплаты заказа
		$order_totals 	= $this->model_checkout_order->getOrderTotals($order["order_id"]);
		$o_shipment 	= 0;
		$o_payment 		= 0;
		$sub_total 		= 0;
		$o_coupon 		= 0;

		foreach ($order_totals as $order_total)
		{
			if ($order_total["code"] == 'shipping') {
				$o_shipment = (float)$order_total["value"];
			}

			if ($order_total["code"] == 'sub_total') {
				$sub_total 	= (float)$order_total["value"];
			}

			if ($order_total["code"] == 'coupon') {
				$o_coupon 	= (float)$order_total["value"];
			}
		}

		// параметры массива
		$o_subtotal = 0;
		$r_products = array();
		$products 	= $this->model_checkout_order->getOrderProducts($order["order_id"]);

		// массив товаров заказа
		foreach ($products as $key_id=>$product)
		{
			// корректируем цену товара с учётом скидок
			$p_price 	 = self::__formatprc(self::__checkProductPrice($product["price"], $sub_total, $o_coupon, $o_payment));
			$o_subtotal += $p_price * $product["quantity"];

			// очищаем имя товара от спец-символов
			$p_name = preg_replace('/[^A-Za-z0-9\-]/', ' ', html_entity_decode($product["name"]));
			$p_name = preg_replace('/ +/', ' ', trim($p_name));

			// формируем массив товаров для чека
			$r_products[$key_id]['name'] 		= $p_name;
			$r_products[$key_id]['price'] 		= $p_price;
			$r_products[$key_id]['quantity'] 	= $product["quantity"];
			$r_products[$key_id]['tax'] 		= $product["tax"];
		}

		// сверка итоговых сумм заказа и чека
		$total_1 	= self::__formatprc($o_subtotal + $o_shipment);
		$total_2 	= self::__formatprc($order["total"]);
		$diff 		= $total_1 != $total_2 ? abs($total_1 - $total_2) : 0;

		// параметры для чека
		$items 		= '';

		// формируем данные чека
		foreach ($r_products as $key_id=>$product)
		{
			// цена товара
			$p_price = $product["price"];

			// изменение цены товара согласно сверки
			if ($diff > 0 && $p_price > $diff) {
				$p_price = $total_2 > $total_1 ? $p_price + ($diff / $product["quantity"]) : $p_price - ($diff / $product["quantity"]);
				$diff 	 = 0;
			}

			// выводим данные товара в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product["name"] . '", ';
			$items .= '"price":' . self::__formatprc($p_price) . ', ';
			$items .= '"quantity":' . $product["quantity"] . ', ';
			$items .= $vat_mode == 1 ? '"vat":"' . $vat . '", ' : '"vat":"VAT_' . floor($product["tax"]) . '", ';
			$items .= '"total":' . self::__formatprc($p_price * $product["quantity"]);
			$items .= '}';
		}

		// доставка
		if ($o_shipment > 0)
		{
			// выводим данные способа доставки в чек
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .= '"calculationMethod":"' . $calc_method . '", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"name":"Доставка", ';
			$items .= '"price":' . self::__formatprc($o_shipment) . ', ';
			$items .= '"quantity":1, ';
			$items .= '"vat":"' . $vat . '", ';
			$items .= '"total":' . self::__formatprc($o_shipment);
			$items .= '}';
		}

		// формируем чек
		$receipt  = '{';
		$receipt .= '"type":"' . $receipt_type . '", "taxSystem":"' . $tax_system . '", ';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $cashier_name . '"';
		$receipt .= $cashier_inn != '' ? ', "inn":"' . self::__checkInn($cashier_inn) . '"' : '';
		$receipt .= '}, ';
		$receipt .= '"client":{';
		$receipt .= $fio ? '"name":"' . trim($order["lastname"] . ' ' . $order["firstname"]) . '", ' : '';
		$receipt .= '"address":"' . $address . '"';
		$receipt .= '}, ';
		$receipt .= '"paymentAddress":"' . $payment_address . '", ';
		$receipt .= '"items":[' . $items . '], ';
		$receipt .= '"total":' . self::__formatprc($order["total"]) . ', ';
		$receipt .= '"payments":{"' . $payment_type . '": ' . self::__formatprc($order["total"]) . '}';
		$receipt .= '}';

		// 3. ОТПРАВКА ЧЕКА В ФИСКАЛЬНЫЙ ПРОЦЕССИНГ
		//    передача контента чека в фискальный процессинг для дальнейшей фискализации

		// адрес для POST-запроса
		$url  		= 'https://fp' . $test . '.bifit.com/processing-api/protected/documents/registration/receipts';

		// данные чека
		$data 		= $receipt;

		// ключ идемпотентности (Idempotency-Key)
		$o_number 	= $order["invoice_prefix"] . $order["invoice_no"];
		$o_summ 	= self::__formatprc($order["total"]);
		$date_time 	= $order["date_added"];
		$idemp_key 	= hash('sha256', "$o_number/$o_summ/$date_time");

		// headers
		$headers 	= array('Content-Type: application/json', 'Authorization: Bearer ' . $bifit->access_token, 'Idempotency-Key: ' . $idemp_key);

		// получаем идентификатор документа на процессинге (id)
		$receipt_id = json_decode(self::__loadData($url, $headers, $data));

		// 4. ПОЛУЧЕНИЕ ДОКУМЕНТА ПО ID
		//    получение документа с фискальными признаками по его номеру

		if (!is_object($receipt_id))
		{
			// ссылка для GET-запроса
			$url 	  = 'https://fp' . $test . '.bifit.com/processing-api/protected/documents/' . $receipt_id;

			// headers
			$headers  = array('Authorization: Bearer ' . $bifit->access_token);

			// получаем документ
			$doc 	  = json_decode(self::__loadData($url, $headers, false));
		}

		// ССЫЛКА НА ЧЕК
		$receipt_link = 'https://fp' . $test . '.bifit.com/processing-api/receipts/' . $idemp_key;

		// ЛОГ ОПЕРАЦИИ

		// Если включено логирование
		if ($log_ok)
		{
			// параметры для лога
			$logObj 				= new stdClass();
			$logObj->order_id 		= $order["order_id"];
			$logObj->order_number 	= $order["invoice_prefix"] . $order["invoice_no"];
			$logObj->order_date 	= $order["date_added"];
			$logObj->cashier 		= $cashier_inn != '' ? $cashier_name . ', ИНН:' . self::__checkInn($cashier_inn) : $cashier_name;
			$logObj->client 		= $fio ? trim($order["lastname"] . ' ' . $order["firstname"]) . ', ' . $address : $address;
			$logObj->receipt_link 	= $receipt_link;
			$logObj->receipt_id 	= !is_object($receipt_id) ? $receipt_id : 'err';
			$doc 					= !is_object($receipt_id) ? $doc : '';
			$logObj->receipt_type 	= $receipt_type;
			$logObj->payment_type 	= $payment_type;

			// сохраняем лог операции
			self::__createLog($test, $bifit, $receipt, $receipt_id, $doc, $receipt_link, $loc, $logObj);
		}
	}

	public function onBeforeDisplayCheckoutFinish($order_data)
	{
		$this->load->model('checkout/order');
		$order 				= $this->model_checkout_order->getOrder($order_data[0]);
		$status_id 			= (int)$order_data[1];
		$order["status_id"] = $status_id;
		$payment_id 		= $order["payment_code"];

		// параметры
		$log_ok 		= $this->config->get('payment_bifit_log');
		$pm_ids 		= (array)$this->config->get('payment_bifit_payment_method');
		$pm_statuses 	= (array)$this->config->get('payment_bifit_order_status_id');
		$test 			= $this->config->get('payment_bifit_test') ? '-test' : '';
		$token 			= $test != '' ? 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE' : $this->config->get('payment_bifit_token');

		// Если совпадает способ оплаты и статус заказа
		if (isset($pm_ids) && isset($pm_statuses) && in_array($payment_id, $pm_ids) && in_array($status_id, $pm_statuses) && $token != '') {

			// запускаем БИФИТ-процесс
			self::__mainProcess($order, $test, $token, $log_ok, 'front');
		}
	}
}
